from psycopg2 import sql
from ..DataStructures.ObjViewEvmTokenHistory import ObjEvmTokenHistory
from ..rupine_db import herokuDbAccess

def ParseObjEvmTokenLatest(data):
    retObj = ObjEvmTokenHistory()
    retObj.token_address = data[0]
    retObj.chain_id = data[1] 
    retObj.abi = data[2]
    retObj.symbol = data[3]
    retObj.name = data[4]
    retObj.decimals = data[5]
    retObj.token_class = data[6]
    retObj.totalsupply = data[7]
    retObj.keywords = data[8]
    retObj.telegram_link = data[9] 
    retObj.creator_address = data[10] 
    retObj.creation_timestamp = data[11] 
    retObj.creation_block_number = data[12] 
    retObj.creation_tx_hash = data[13] 
    retObj.max_tx_amount_percent = data[14]
    retObj.max_wallet_size_percent = data[15]
    retObj.is_verified = data[16]
    retObj.is_honeypot = data[17]
    retObj.buy_tax = data[18]
    retObj.sell_tax = data[19]
    retObj.latest_timestamp = data[20]
    return retObj

def getEvmTokenLatest(connection, schema, chain_id, token_address):
    
    # query database    
    query = sql.SQL("SELECT token_address, chain_id, abi, symbol, name, decimals, token_class, totalsupply, keywords, telegram_link, creator_address, creation_timestamp, creation_block_number, creation_tx_hash, max_tx_amount_percent, max_wallet_size_percent, is_verified, is_honeypot, buy_tax, sell_tax, latest_timestamp \
        FROM {}.v_evm_latest_token WHERE chain_id = %s AND token_address = %s").format(sql.Identifier(schema))
    result = herokuDbAccess.fetchDataInDatabase(query, [chain_id,token_address], connection)    
    
    # parse into objects
    rows = []
    for tok in result:
        addRow = ParseObjEvmTokenLatest(tok)
        rows.append(addRow)

    # return objects
    return rows

def getEvmTokenLatestList(connection, schema, chain_id, gteCreatedAt):
    
    # query database    
    query = sql.SQL("SELECT token_address, chain_id, abi, symbol, name, decimals, token_class, totalsupply, keywords, telegram_link, creator_address, creation_timestamp, creation_block_number, creation_tx_hash, max_tx_amount_percent, max_wallet_size_percent, is_verified, is_honeypot, buy_tax, sell_tax, latest_timestamp \
        FROM {}.v_evm_latest_token WHERE chain_id = %s AND creation_timestamp >= %s").format(sql.Identifier(schema))
    result = herokuDbAccess.fetchDataInDatabase(query, [chain_id,gteCreatedAt], connection)    
    
    # parse into objects
    rows = []
    for tok in result:
        addRow = ParseObjEvmTokenLatest(tok)
        rows.append(addRow)

    # return objects
    return rows

def getEvmTokenLatestWithoutName(connection, schema, chain_id, gteCreatedAt):
    
    # query database    
    query = sql.SQL("SELECT token_address, chain_id, abi, symbol, name, decimals, token_class, totalsupply, keywords, telegram_link, creator_address, creation_timestamp, creation_block_number, creation_tx_hash, max_tx_amount_percent, max_wallet_size_percent, is_verified, is_honeypot, buy_tax, sell_tax, latest_timestamp \
        FROM {}.v_evm_latest_token WHERE chain_id = %s AND (name is NULL OR name = 'n/a') AND creation_timestamp >= %s").format(sql.Identifier(schema))
    result = herokuDbAccess.fetchDataInDatabase(query, [chain_id,gteCreatedAt], connection)    
    
    # parse into objects
    rows = []
    for tok in result:
        addRow = ParseObjEvmTokenLatest(tok)
        rows.append(addRow)

    # return objects
    return rows



class ObjViewEvmTokenHistory:
    token_address = None
    abi = None
    chain_id = None 
    symbol = None
    name = None
    decimals = None
    token_class = None
    totalsupply = None
    keywords = None
    telegram_link = None
    creator_address = None
    creation_timestamp = None 
    creation_block_number = None 
    creation_tx_hash = None
    max_tx_amount_percent = None
    max_wallet_size_percent = None
    is_verified = None
    is_honeypot = None
    buy_tax = None
    sell_tax = None
    latest_timestamp = 0

    def __init__(self):
        pass

    def __eq__(self, other): 
        if not isinstance(other, ObjViewEvmTokenHistory):
            # don't attempt to compare against unrelated types
            return NotImplemented

        return (self.token_address == other.token_address and 
                self.abi == other.abi and
                self.chain_id == other.chain_id and
                self.symbol == other.symbol and
                self.name == other.name and
                self.decimals == other.decimals and
                self.token_class == other.token_class and
                self.totalsupply == other.totalsupply and
                self.keywords == other.keywords and
                self.telegram_link == other.telegram_link and
                self.creator_address == other.creator_address and
                self.creation_timestamp == other.creation_timestamp and
                self.creation_block_number == other.creation_block_number and
                self.creation_tx_hash == other.creation_tx_hash and
                self.max_tx_amount_percent == other.max_tx_amount_percent and
                self.max_wallet_size_percent == other.max_wallet_size_percent and
                self.is_verified == other.is_verified and
                self.is_honeypot == other.is_honeypot and
                self.buy_tax == other.buy_tax and
                self.sell_tax == other.sell_tax and
                self.latest_timestamp == other.latest_timestamp)

    def clone(self):
        retTok = ObjViewEvmTokenHistory()
        retTok.token_address = self.token_address
        retTok.abi = self.abi
        retTok.chain_id = self.chain_id
        retTok.symbol = self.symbol
        retTok.name = self.name
        retTok.decimals = self.decimals
        retTok.token_class = self.token_class
        retTok.totalsupply = self.totalsupply
        retTok.keywords = self.keywords
        retTok.telegram_link = self.telegram_link
        retTok.creator_address = self.creator_address
        retTok.creation_timestamp = self.creation_timestamp
        retTok.creation_block_number = self.creation_block_number
        retTok.creation_tx_hash = self.creation_tx_hash
        retTok.max_tx_amount_percent = self.max_tx_amount_percent
        retTok.max_wallet_size_percent = self.max_wallet_size_percent
        retTok.is_verified = self.is_verified
        retTok.is_honeypot = self.is_honeypot
        retTok.buy_tax = self.buy_tax
        retTok.sell_tax = self.sell_tax
        retTok.latest_timestamp = self.latest_timestamp
        return retTok


from psycopg2 import sql
from ..rupine_db import herokuDbAccess

class ObjDmhNetworkStatisticHistory:
    network_no = ''
    gecko_market_cap_rank = 0
    gecko_market_cap = 0
    gecko_current_price = 0
    gecko_total_volume = 0
    twitter_follower = 0
    twitter_hashtag_count = 0
    github_total_commits = 0
    github_contributer_gte_100 = 0
    github_contributer_lt_100_gte_10 = 0
    github_contributer_lt_10 = 0
    github_contributer_monthly_gte_100 = 0
    github_contributer_monthly_lt_100 = 0
    defilama_tvl = 0
    created_at = 0 
    modified_at= 0
    reddit_subscribers = 0
    reddit_average_posts_48h = 0
    reddit_average_comments_48h = 0
    reddit_accounts_active_48h = 0
    github_forks = 0
    github_stars = 0
    github_subscribers = 0
    github_pull_requests_merged = 0
    github_pull_request_contributors = 0
    github_commit_count_4_weeks = 0
    timestamp = 0

def ParseDataIntoObj(data):
    retObj = ObjDmhNetworkStatisticHistory()
    retObj.network_no = data[0]
    retObj.gecko_market_cap_rank = data[1]
    retObj.gecko_market_cap = data[2]
    retObj.gecko_current_price = data[3]
    retObj.gecko_total_volume = data[4]
    retObj.twitter_follower = data[5]
    retObj.twitter_hashtag_count = data[6]
    retObj.github_total_commits = data[7]
    retObj.github_contributer_gte_100 = data[8]
    retObj.github_contributer_lt_100_gte_10 = data[9]
    retObj.github_contributer_lt_10 = data[10]
    retObj.github_contributer_monthly_gte_100 = data[10]
    retObj.github_contributer_monthly_lt_100 = data[11]
    retObj.defilama_tvl = data[12]
    retObj.created_at = data[13]
    retObj.modified_at= data[14]
    retObj.reddit_subscribers = data[15]
    retObj.reddit_average_posts_48h = data[16]
    retObj.reddit_average_comments_48h = data[17]
    retObj.reddit_accounts_active_48h = data[18]
    retObj.github_forks = data[19]
    retObj.github_stars = data[20]
    retObj.github_subscribers = data[21]
    retObj.github_pull_requests_merged = data[22]
    retObj.github_pull_request_contributors = data[23]
    retObj.github_commit_count_4_weeks = data[24]
    retObj.timestamp = data[25]
    return retObj

def postHistoryEntry(connection, schema:str, entry:ObjDmhNetworkStatisticHistory):

    query = sql.SQL("INSERT INTO {}.dwh_network_statistics_history (network_no, gecko_market_cap_rank, gecko_market_cap, gecko_current_price, gecko_total_volume, twitter_follower, twitter_hashtag_count, github_total_commits, github_contributer_gte_100, github_contributer_lt_100_gte_10, github_contributer_lt_10, github_contributer_monthly_gte_100, github_contributer_monthly_lt_100, defilama_tvl, reddit_subscribers, reddit_average_posts_48h, reddit_average_comments_48h, reddit_accounts_active_48h, github_forks, github_stars, github_subscribers, github_pull_requests_merged, github_pull_request_contributors, github_commit_count_4_weeks, timestamp) \
           VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))
    params = (
        entry.network_no,
        entry.gecko_market_cap_rank,
        entry.gecko_market_cap,
        entry.gecko_current_price,
        entry.gecko_total_volume,
        entry.twitter_follower,
        entry.twitter_hashtag_count,
        entry.github_total_commits,
        entry.github_contributer_gte_100,
        entry.github_contributer_lt_100_gte_10,
        entry.github_contributer_lt_10,
        entry.github_contributer_monthly_gte_100,
        entry.github_contributer_monthly_lt_100,
        entry.defilama_tvl,
        entry.reddit_subscribers,
        entry.reddit_average_posts_48h,
        entry.reddit_average_comments_48h,
        entry.reddit_accounts_active_48h,
        entry.github_forks,
        entry.github_stars,
        entry.github_subscribers,
        entry.github_pull_requests_merged,
        entry.github_pull_request_contributors,
        entry.github_commit_count_4_weeks,
        entry.timestamp)

    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
    return result

def getNetworkHistory(connection, schema, network_no):
    
    # query database    
    query = sql.SQL("SELECT network_no, gecko_market_cap_rank, gecko_market_cap, gecko_current_price, gecko_total_volume, \
            twitter_follower, twitter_hashtag_count, github_total_commits, github_contributer_gte_100, \
            github_contributer_lt_100_gte_10, github_contributer_lt_10, github_contributer_monthly_gte_100, \
            github_contributer_monthly_lt_100, defilama_tvl, created_at, modified_at, reddit_subscribers, \
            reddit_average_posts_48h, reddit_average_comments_48h, reddit_accounts_active_48h, github_forks, \
            github_stars, github_subscribers, github_pull_requests_merged, github_pull_request_contributors, \
            github_commit_count_4_weeks, timestamp \
        FROM {}.dwh_network_statistics_history WHERE network_no=%s").format(sql.Identifier(schema))
    result = herokuDbAccess.fetchDataInDatabase(query, [network_no], connection)    
    
    # parse into objects
    rows = []
    for tok in result:
        addRow = ParseDataIntoObj(tok)
        rows.append(addRow)

    # return objects
    return rows

def getNetworkHistory(connection, schema, network_no, time_start, time_end):
    
    # query database    
    query = sql.SQL("SELECT network_no, gecko_market_cap_rank, gecko_market_cap, gecko_current_price, gecko_total_volume, \
            twitter_follower, twitter_hashtag_count, github_total_commits, github_contributer_gte_100, \
            github_contributer_lt_100_gte_10, github_contributer_lt_10, github_contributer_monthly_gte_100, \
            github_contributer_monthly_lt_100, defilama_tvl, created_at, modified_at, reddit_subscribers, \
            reddit_average_posts_48h, reddit_average_comments_48h, reddit_accounts_active_48h, github_forks, \
            github_stars, github_subscribers, github_pull_requests_merged, github_pull_request_contributors, \
            github_commit_count_4_weeks, timestamp \
        FROM {}.dwh_network_statistics_history \
        WHERE network_no=%s AND timestamp>%s AND timestamp<%s").format(sql.Identifier(schema))
    result = herokuDbAccess.fetchDataInDatabase(query, [network_no, time_start, time_end], connection)    
    
    # parse into objects
    rows = []
    for tok in result:
        addRow = ParseDataIntoObj(tok)
        rows.append(addRow)

    # return objects
    return rows


from psycopg2 import sql
from ..rupine_db import herokuDbAccess

def postKeywords(connection, schema:str, keyword, keyword_detail, reference, method, value, message):

    query = sql.SQL("INSERT INTO {}.keyword_config (keyword, keyword_detail, reference, method, value, message) \
           VALUES (%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))
    params = (
        keyword,
        keyword_detail,
        reference,
        method,
        value,
        message
    )
    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
    return result

def getKeywords(connection, schema):
    
    query = sql.SQL("SELECT * FROM {}.keyword_config").format(sql.Identifier(schema))
    
    params =()
    
    result = herokuDbAccess.fetchDataInDatabase(query, params, connection)    
    return result
 
    